/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      XMEGA USART driver source file.
 *
 *      This file contains the function implementations the XMEGA interrupt
 *      and polled USART driver.
 *
 *      The driver is not intended for size and/or speed critical code, since
 *      most functions are just a few lines of code, and the function call
 *      overhead would decrease code performance. The driver is intended for
 *      rapid prototyping and documentation purposes for getting started with
 *      the XMEGA ADC module.
 *
 *      For size and/or speed critical code, it is recommended to copy the
 *      function contents directly into your application instead of making
 *      a function call.
 *
 *      Some functions use the following construct:
 *          "some_register = ... | (some_parameter ? SOME_BIT_bm : 0) | ..."
 *      Although the use of the ternary operator ( if ? then : else ) is discouraged,
 *      in some occasions the operator makes it possible to write pretty clean and
 *      neat code. In this driver, the construct is used to set or not set a
 *      configuration bit based on a boolean input parameter, such as
 *      the "some_parameter" in the example above.
 *
 * \par Application note:
 *      AVR1307: Using the XMEGA USART
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 1694 $
 * $Date: 2008-07-29 14:21:58 +0200 (ti, 29 jul 2008) $  \n
 *
 * Copyright (c) 2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
#include "usart_driver.h"

#define USART_RX_BUFFER_MASK ( USART_RX_BUFFER_SIZE - 1 )		// Receive buffer mask. 
#define USART_TX_BUFFER_MASK ( USART_TX_BUFFER_SIZE - 1 )		// Transmit buffer mask. 


#if ( USART_RX_BUFFER_SIZE & USART_RX_BUFFER_MASK )
#error RX buffer size is not a power of 2
	#endif
#if ( USART_TX_BUFFER_SIZE & USART_TX_BUFFER_MASK )
	#error TX buffer size is not a power of 2
#endif

/*! \brief Test if there is data in the transmitter software buffer.
 *
 *  This function can be used to test if there is free space in the transmitter
 *  software buffer.
 *
 *  \param usart_data The USART_data_t struct instance.
 *
 *  \retval true      There is data in the receive buffer.
 *  \retval false     The receive buffer is empty.
 */
bool USART_TXBuffer_FreeSpace(USART_data_t * usart_data)
{
	/* Make copies to make sure that volatile access is specified. */
	uint8_t tempHead = (usart_data->buffer.TX_Head + 1) & USART_TX_BUFFER_MASK;
	uint8_t tempTail = usart_data->buffer.TX_Tail;

	/* There are data left in the buffer unless Head and Tail are equal. */
	return (tempHead != tempTail);
}
/*! \brief Put data (5-8 bit character).
 *
 *  Stores data byte in TX software buffer and enables DRE interrupt if there
 *  is free space in the TX software buffer.
 *
 *  \param usart_data The USART_data_t struct instance.
 *  \param data       The data to send.
 */
bool USART_TXBuffer_PutByte(USART_data_t * usart_data, uint8_t data)
{
	uint8_t tempCTRLA;
	uint8_t tempTX_Head;
	bool TXBuffer_FreeSpace;
	USART_Buffer_t * TXbufPtr;

	TXbufPtr = &usart_data->buffer;
	TXBuffer_FreeSpace = USART_TXBuffer_FreeSpace(usart_data);


	if(TXBuffer_FreeSpace)
	{
	  	tempTX_Head = TXbufPtr->TX_Head;
	  	TXbufPtr->TX[tempTX_Head]= data;
		/* Advance buffer head. */
		TXbufPtr->TX_Head = (tempTX_Head + 1) & USART_TX_BUFFER_MASK;

		/* Enable DRE interrupt. */
		tempCTRLA = usart_data->usart->CTRLA;
		tempCTRLA = (tempCTRLA & ~USART_DREINTLVL_gm) | usart_data->dreIntLevel;
		usart_data->usart->CTRLA = tempCTRLA;
	}
	return TXBuffer_FreeSpace;
}
/*! \brief Test if there is data in the receive software buffer.
 *
 *  This function can be used to test if there is data in the receive software
 *  buffer.
 *
 *  \param usart_data         The USART_data_t struct instance
 *
 *  \retval true      There is data in the receive buffer.
 *  \retval false     The receive buffer is empty.
 */
bool USART_RXBufferData_Available(USART_data_t * usart_data)
{
	/* Make copies to make sure that volatile access is specified. */
	uint8_t tempHead = usart_data->buffer.RX_Head;
	uint8_t tempTail = usart_data->buffer.RX_Tail;

	/* There are data left in the buffer unless Head and Tail are equal. */
	return (tempHead != tempTail);
}
/*! \brief Get received data (5-8 bit character).
 *
 *  The function USART_RXBufferData_Available should be used before this
 *  function is used to ensure that data is available.
 *
 *  Returns data from RX software buffer.
 *
 *  \param usart_data       The USART_data_t struct instance.
 *
 *  \return         Received data.
 */
char USART_RXBuffer_GetByte(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr;
	char ans;

	bufPtr = &usart_data->buffer;
	ans = (bufPtr->RX[bufPtr->RX_Tail]);

	/* Advance buffer tail. */
	bufPtr->RX_Tail = (bufPtr->RX_Tail + 1) & USART_RX_BUFFER_MASK;

	return ans;
}
/*! \brief RX Complete Interrupt Service Routine.
 *
 *  RX Complete Interrupt Service Routine.
 *  Stores received data in RX software buffer.
 *
 *  \param usart_data      The USART_data_t struct instance.
 */
bool USART_RXComplete(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr;
	bool ans;

	bufPtr = &usart_data->buffer;
	/* Advance buffer head. */
	uint8_t tempRX_Head = (bufPtr->RX_Head + 1) & USART_RX_BUFFER_MASK;

	/* Check for overflow. */
	uint8_t tempRX_Tail = bufPtr->RX_Tail;
	uint8_t data = usart_data->usart->DATA;

	if (tempRX_Head == tempRX_Tail) 
	{
	  	ans = false;
	}
	else
	{
		ans = true;
		usart_data->buffer.RX[usart_data->buffer.RX_Head] = data;
		usart_data->buffer.RX_Head = tempRX_Head;
	}
	return ans;
}
/*! \brief Data Register Empty Interrupt Service Routine.
 *
 *  Data Register Empty Interrupt Service Routine.
 *  Transmits one byte from TX software buffer. Disables DRE interrupt if buffer
 *  is empty. Argument is pointer to USART (USART_data_t).
 *
 *  \param usart_data      The USART_data_t struct instance.
 */
void USART_DataRegEmpty(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr;
	bufPtr = &usart_data->buffer;

	/* Check if all data is transmitted. */
	uint8_t tempTX_Tail = usart_data->buffer.TX_Tail;
	if (bufPtr->TX_Head == tempTX_Tail){
	    /* Disable DRE interrupts. */
		uint8_t tempCTRLA = usart_data->usart->CTRLA;
		tempCTRLA = (tempCTRLA & ~USART_DREINTLVL_gm) | USART_DREINTLVL_OFF_gc;
		usart_data->usart->CTRLA = tempCTRLA;

	}
	else
	{
		/* Start transmitting. */
		char data = bufPtr->TX[usart_data->buffer.TX_Tail];
		usart_data->usart->DATA = data;

		/* Advance buffer tail. */
		bufPtr->TX_Tail = (bufPtr->TX_Tail + 1) & USART_TX_BUFFER_MASK;
	}
}
/*! \brief Put data (9 bit character).
 *
 *  Use the function USART_IsTXDataRegisterEmpty before using this function to
 *  put 9 bit character to the TX register.
 *
 *  \param usart      The USART module.
 *  \param data       The data to send.
 */
void USART_NineBits_PutChar(USART_t * usart, uint16_t data)
{
	if(data & 0x0100) {
		usart->CTRLB |= USART_TXB8_bm;
	}else {
		usart->CTRLB &= ~USART_TXB8_bm;
	}

	usart->DATA = (data & 0x00FF);
}
/*! \brief Get received data (9 bit character).
 *
 *  This function reads out the received 9 bit character (uint16_t).
 *  Use the function USART_IsRXComplete to check if anything is received.
 *
 *  \param usart      The USART module.
 *
 *  \retval           Received data.
 */
uint16_t USART_NineBits_GetChar(USART_t * usart)
{
	if(usart->CTRLB & USART_RXB8_bm) 
	{
		return(0x0100 | usart->DATA);
	}
	else 
	{
		return(usart->DATA);
	}
}

//#######################################################################################
//#######################################################################################
#ifdef ENABLE_XONXOFF
	volatile uint8_t flow_ctrl = XON_SENT; // Flow control state variable
  
	// Returns the number of bytes in the RX buffer. This replaces a typical byte counter to prevent
	// the interrupt and main programs from writing to the counter at the same time.

#endif

//#######################################################################################
void usart_init(USART_data_t * usart_data, USART_t * usart, unsigned int baudrate, USART_DREINTLVL_t dreIntLevel)
{
	// Purpose:  initialize UART and set baudrate
	// Input:    baudrate using macro UART_BAUD_SELECT()
	// Returns:  none
	
	usart_data->usart = usart;
	usart_data->dreIntLevel = dreIntLevel;
	
	usart_data->buffer.RX_Tail = 0;
	usart_data->buffer.RX_Head = 0;
	usart_data->buffer.TX_Tail = 0;
	usart_data->buffer.TX_Head = 0;
			
	USART_Format_Set(usart_data->usart, USART_CHSIZE_8BIT_gc, USART_PMODE_DISABLED_gc, false);	// USARTC0, 8 Data bits, No Parity, 1 Stop bit. 
	USART_RxdInterruptLevel_Set(usart_data->usart, USART_RXCINTLVL_LO_gc);						// Enable RXC interrupt. 
	
	if (baudrate & 0x8000)						
	{
		USART_CLK2X_Enable(usart);				// Double Speed Modus
		baudrate &= ~0x8000;
		USART_Baudrate_Set(usart, baudrate , 0); 
	}		
	else
	{
		USART_Baudrate_Set(usart, baudrate , 0); 
	}
	
	USART_Rx_Enable(usart_data->usart);			// Enable RX
	USART_Tx_Enable(usart_data->usart);			// Enable TX
}
//#######################################################################################
void usart_rxbuffer_reset(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr = &usart_data->buffer;
	bufPtr->RX_Tail = bufPtr->RX_Head;

	#ifdef ENABLE_XONXOFF
		flow_ctrl = XON_SENT;
	#endif	
}
//#######################################################################################
void usart_txbuffer_reset(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr = &usart_data->buffer;
	bufPtr->TX_Tail = bufPtr->TX_Head;
}
//#######################################################################################
uint8_t usart_rxbuffer_freespace(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr = &usart_data->buffer;
		
	if (bufPtr->RX_Head == bufPtr->RX_Tail)		return(0); 
	if (bufPtr->RX_Head  < bufPtr->RX_Tail)		return(bufPtr->RX_Tail - bufPtr->RX_Head);			
	return (USART_RX_BUFFER_SIZE - (bufPtr->RX_Head - bufPtr->RX_Tail));
}
//#######################################################################################
uint8_t usart_txbuffer_freespace(USART_data_t * usart_data)
{
	USART_Buffer_t * bufPtr = &usart_data->buffer;
		
	if (bufPtr->TX_Head == bufPtr->TX_Tail)		return(0); 
	if (bufPtr->TX_Head  < bufPtr->TX_Tail)		return(bufPtr->RX_Tail - bufPtr->RX_Head);			
	return (USART_TX_BUFFER_SIZE - (bufPtr->TX_Head - bufPtr->TX_Tail));
}
//#######################################################################################
void usart_rx(USART_data_t * usart_data)
{
	// brief RX Complete Interrupt Service Routine.
	// RX Complete Interrupt Service Routine.
	// Stores received data in RX software buffer.
	// param usart_data      The USART_data_t struct instance.
 	
	
    unsigned char lastRxError;	 
	USART_Buffer_t * bufPtr;

	bufPtr = &usart_data->buffer;
	
    uint8_t status  = usart_data->usart->STATUS;
    uint8_t data	= usart_data->usart->DATA;	

	//USART_PERR_bm		// bit 2 PERR:  Parity Error
	//USART_BUFOVF_bm	// bit 3 BUFOVF:  Buffer Overflow
	//USART_FERR_bm		// bit 4 FERR:  Frame Error
	 lastRxError = (status &  (1<<USART_FERR_bm)|(1<<USART_BUFOVF_bm)|(1<<USART_PERR_bm) );
	//bei atmega lastRxError = (status &  (1<<FE)|(1<<DOR)|(1<<UPE) );	// Bit 4  FEn: Frame Error, Bit 3  DORn: Data OverRun, Bit 2  UPEn: Parity Error
	
	uint16_t tempRX_Head = (bufPtr->RX_Head + 1) & USART_RX_BUFFER_MASK;			// calculate buffer index

	if (tempRX_Head == bufPtr->RX_Tail) 
	{
	  	// error: receive buffer overflow 
        lastRxError = USART_BUFFER_OVERFLOW >> 8;
	}
	else
	{
        usart_data->buffer.RX_Head = tempRX_Head;						// store new index        
		usart_data->buffer.RX[usart_data->buffer.RX_Head] = data;		// store received data in buffer 
		
        #ifdef ENABLE_XONXOFF
			if ((usart_rxbuffer_freespace(usart_data) >= RX_BUFFER_FULL) && flow_ctrl == XON_SENT) 
			{
				flow_ctrl = SEND_XOFF;
				// Force TX
				uint8_t tempCTRLA = usart_data->usart->CTRLA;
				tempCTRLA = (tempCTRLA & ~USART_DREINTLVL_gm) | usart_data->dreIntLevel;
				usart_data->usart->CTRLA = tempCTRLA;
			} 
        #endif		
	}
	

	
	usart_data->LastRxError = lastRxError;
}
//#######################################################################################
void usart_tx(USART_data_t * usart_data)
{
	// Function: UART Data Register Empty interrupt
	// Purpose : called when the UART is ready to transmit the next byte	
	// brief Data Register Empty Interrupt Service Routine.
	// Data Register Empty Interrupt Service Routine.
	// Transmits one byte from TX software buffer. Disables DRE interrupt if buffer
	// is empty. Argument is pointer to USART (USART_data_t).
	// param usart_data      The USART_data_t struct instance.
	
	USART_Buffer_t * bufPtr;
	bufPtr = &usart_data->buffer;

	// Check if all data is transmitted. 
	uint16_t tempTX_Tail = usart_data->buffer.TX_Tail;
	
	#ifdef ENABLE_XONXOFF
		if (flow_ctrl == SEND_XOFF) 
		{ 
			usart_data->usart->DATA = XOFF_CHAR; 
			flow_ctrl = XOFF_SENT; 
		} 
		else if (flow_ctrl == SEND_XON) 
		{ 
			usart_data->usart->DATA = XON_CHAR; 
			flow_ctrl = XON_SENT; 
		} else
	#endif
	
	if (bufPtr->TX_Head == tempTX_Tail)
	{
	    // Disable DRE interrupts. 
		char tempCTRLA = usart_data->usart->CTRLA;
		tempCTRLA = (tempCTRLA & ~USART_DREINTLVL_gm) | USART_DREINTLVL_OFF_gc;
		usart_data->usart->CTRLA = tempCTRLA;
	}
	else
	{
		bufPtr->TX_Tail = (bufPtr->TX_Tail + 1) & USART_TX_BUFFER_MASK;			// calculate and store new buffer index 		
		usart_data->usart->DATA = bufPtr->TX[usart_data->buffer.TX_Tail];		// get one byte from buffer and write it to UART 
	}	
}
//#######################################################################################
void usart_putc(USART_data_t * usart_data, const char data)
{
	// Purpose:  Stores data byte in TX software buffer and enables DRE interrupt if there
	//			 is free space in the TX software buffer.
	// Input:    Put data (5-8 bit character).
	// Returns:  TXBuffer_FreeSpace 
	
	USART_Buffer_t * TXbufPtr = &usart_data->buffer;	
	uint16_t tempTX_Head;
	
    while (!USART_TXBuffer_FreeSpace(usart_data))// tempTX_Head == TXbufPtr->TX_Tail) // todo
	{
		; // wait for free space in buffer 
    }

	tempTX_Head = TXbufPtr->TX_Head;
	TXbufPtr->TX[tempTX_Head]= data;
	TXbufPtr->TX_Head = (tempTX_Head + 1) & USART_TX_BUFFER_MASK;	// calculate tx buffer head.
	
	// Enable DRE interrupt. 
	uint8_t tempCTRLA = usart_data->usart->CTRLA;
	tempCTRLA = (tempCTRLA & ~USART_DREINTLVL_gm) | usart_data->dreIntLevel;
	usart_data->usart->CTRLA = tempCTRLA;
	
	
}
//#######################################################################################     
void usart_puts(USART_data_t * usart_data, const char *str )
{	
	// Purpose:  transmit string to UART1
	// Input:    string to be transmitted
	// Returns:  TXBuffer_FreeSpace    	
	
	while (*str) 
	{		
		usart_putc(usart_data,*str++);
	}			
}
//#######################################################################################
void usart_puts_p(USART_data_t * usart_data, const char *progmem_str )  
{
	// Print a string stored in PGM-memory
	// Purpose:  transmit string from program memory to UART1
	// Input:    program memory string to be transmitted
	// Returns:  TXBuffer_FreeSpace		
	
	register char c;
	while ((c = pgm_read_byte(progmem_str++)))			// pgm_read_byte_near
	{
		usart_putc(usart_data,c);
	}			
}
//#######################################################################################
unsigned int usart_getc(USART_data_t * usart_data)
{
	// Purpose:  get Received data, use USART_RXBufferData_Available before
	// Input:    no input
	// Returns:  Received data
	
	USART_Buffer_t * bufPtr = &usart_data->buffer;
	uint8_t tmptail;
	uint8_t data;

	if (bufPtr->RX_Head == bufPtr->RX_Tail)		return USART_NO_DATA;	// no data available 
	
	// calculate /store buffer index 
	tmptail = (bufPtr->RX_Tail + 1) & USART_RX_BUFFER_MASK;
	bufPtr->RX_Tail = tmptail;
	
	// get data from receive buffer 
	bufPtr = &usart_data->buffer;
	data = bufPtr->RX[tmptail];
	
	#ifdef ENABLE_XONXOFF
		if ((usart_rxbuffer_freespace(usart_data) < RX_BUFFER_LOW) && flow_ctrl == XOFF_SENT) 
		{ 
			flow_ctrl = SEND_XON;
			// Force TX
			uint8_t tempCTRLA = usart_data->usart->CTRLA;
			tempCTRLA = (tempCTRLA & ~USART_DREINTLVL_gm) | usart_data->dreIntLevel;
			usart_data->usart->CTRLA = tempCTRLA;
		}		
    #endif
	
	return data;//(usart_data->LastRxError << 8) + data;	
}


//##########################################################################################
// Spezial Funktionen
//#######################################################################################
#if EXTENDED
//##########################################################################################
void usart_putc_hex(USART_data_t * usart_data,uint8_t b)
{
    /* upper nibble */
    if((b >> 4) < 0x0a)
        usart_putc(usart_data,(b >> 4) + '0');
    else
        usart_putc(usart_data,(b >> 4) - 0x0a + 'a');

    /* lower nibble */
    if((b & 0x0f) < 0x0a)
        usart_putc(usart_data,(b & 0x0f) + '0');
    else
        usart_putc(usart_data,(b & 0x0f) - 0x0a + 'a');
}
//##########################################################################################
void usart_putw_hex(USART_data_t * usart_data,uint16_t w)
{
    usart_putc_hex(usart_data,(uint8_t) (w >> 8));
    usart_putc_hex(usart_data,(uint8_t) (w & 0xff));
}
//##########################################################################################
void usart_putdw_hex(USART_data_t * usart_data,uint32_t dw)
{
    usart_putw_hex(usart_data,(uint16_t) (dw >> 16));
    usart_putw_hex(usart_data,(uint16_t) (dw & 0xffff));
}
//##########################################################################################
void usart_putw_dec(USART_data_t * usart_data,uint16_t w)
{
    uint16_t num = 10000;
    uint8_t started = 0;

    while(num > 0)
    {
        uint8_t b = w / num;
        if(b > 0 || started || num == 1)
        {
            usart_putc(usart_data,'0' + b);
            started = 1;
        }
        w -= b * num;

        num /= 10;
    }
}
//##########################################################################################
void usart_putdw_dec(USART_data_t * usart_data,uint32_t dw)
{
    uint32_t num = 1000000000;
    uint8_t started = 0;

    while(num > 0)
    {
        uint8_t b = dw / num;
        if(b > 0 || started || num == 1)
        {
            usart_putc(usart_data,'0' + b);
            started = 1;
        }
        dw -= b * num;

        num /= 10;
    }
}
//##########################################################################################
void printIntegerInBase(USART_data_t * usart_data,unsigned long n, unsigned long base)
{ 
	unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
	unsigned long i = 0;

	if (n == 0) 
	{
		usart_putc(usart_data,'0');
		return;
	} 

	while (n > 0) 
	{
		buf[i++] = n % base;
		n /= base;
	}

	for (; i > 0; i--)
		usart_putc(usart_data,buf[i - 1] < 10 ?
			'0' + buf[i - 1] :
			'A' + buf[i - 1] - 10);
}
//##########################################################################################
void printInteger(USART_data_t * usart_data,long n)
{
	if (n < 0) 
	{
		usart_putc(usart_data,'-');
		n = -n;
	}

	printIntegerInBase(usart_data,n, 10);
}
//##########################################################################################
void printFloat(USART_data_t * usart_data,double n)
{
  double integer_part, fractional_part;
	if (n < 0) 
		{
		usart_putc(usart_data,'-');
		n=-n;
		} 
  
  fractional_part = modf(n, &integer_part);
  printInteger(usart_data, integer_part);
  usart_putc(usart_data, '.');
  printInteger(usart_data, round(fractional_part*1000));
}
//##########################################################################################
#endif
//#######################################################################################
// void printIntegerInBase(unsigned long n, unsigned long base)
// { 
// 	unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
// 	unsigned long i = 0;
// 
// 	if (n == 0) {
// 		usart_putc(&USART_PC_data,'0');
// 		return;
// 	} 
// 
// 	while (n > 0) {
// 		buf[i++] = n % base;
// 		n /= base;
// 	}
// 
// 	for (; i > 0; i--)
// 		usart_putc(&USART_PC_data,buf[i - 1] < 10 ?
// 			'0' + buf[i - 1] :
// 			'A' + buf[i - 1] - 10);
// }

#if EXTENDED
//#######################################################################################
void print_uint8_base2(uint8_t n)
{ 
	unsigned char buf[8];
	uint8_t i = 0;

	for (; i < 8; i++) {
		buf[i] = n & 1;
		n >>= 1;
	}

	for (; i > 0; i--)
		usart_putc(&USART_PC_data,'0' + buf[i - 1]);
}
//#######################################################################################
static void print_uint32_base10(unsigned long n)
{ 
  unsigned char buf[10]; 
  uint8_t i = 0;
  
  if (n == 0) {
    usart_putc(&USART_PC_data,'0');
    return;
  } 
  
  while (n > 0) {
    buf[i++] = n % 10 + '0';
    n /= 10;
  }
    
  for (; i > 0; i--)
    usart_putc(&USART_PC_data,buf[i-1]);
}
//#######################################################################################
void printInteger(long n)
{
  if (n < 0) {
    usart_putc(&USART_PC_data,'-');
    n = -n;
  }
  print_uint32_base10(n);
}
//#######################################################################################
// Convert float to string by immediately converting to a long integer, which contains
// more digits than a float. Number of decimal places, which are tracked by a counter,
// may be set by the user. The integer is then efficiently converted to a string.
// NOTE: AVR '%' and '/' integer operations are very efficient. Bitshifting speed-up 
// techniques are actually just slightly slower. Found this out the hard way.
void printFloat(float n)
{
  if (n < 0) {
    usart_putc(&USART_PC_data,'-');
    n = -n;
  }

  uint8_t decimals = settings.decimal_places;
  while (decimals >= 2) { // Quickly convert values expected to be E0 to E-4.
    n *= 100;
    decimals -= 2;
  }
  if (decimals) { n *= 10; }
  n += 0.5; // Add rounding factor. Ensures carryover through entire value.
    
  // Generate digits backwards and store in string.
  unsigned char buf[10]; 
  uint8_t i = 0;
  uint32_t a = (long)n;  
  buf[settings.decimal_places] = '.'; // Place decimal point, even if decimal places are zero.
  while(a > 0) {
    if (i == settings.decimal_places) { i++; } // Skip decimal point location
    buf[i++] = (a % 10) + '0'; // Get digit
    a /= 10;
  }
  while (i < settings.decimal_places) { 
     buf[i++] = '0'; // Fill in zeros to decimal point for (n < 1)
  }
  if (i == settings.decimal_places) { // Fill in leading zero, if needed.
    i++;
    buf[i++] = '0'; 
  }   
  
  // Print the generated string.
  for (; i > 0; i--)
    usart_putc(&USART_PC_data,buf[i-1]);
}
//#######################################################################################

#endif