/*#################################################################################################
	For		:	Xmega 
	Version	:	1.0
	Date	:	30.07.2013
	Autor	:	Martin Junghans
	Hardware:	ATXmega, 32MHz at 16Mhz extern, Wiznet, Webserver
//################################################################################################*/

#ifndef WIZNET_DRIVER_H
#define WIZNET_DRIVER_H

#include <stdlib.h>
#include <avr/io.h>
#include <stdio.h>
#include <stddef.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "spi_driver.h"
#include "macros.h"

//########################################################################################## Buffer
#define RX_BUF_MAX						1024	// 512 byte  1024
#define NET_MEMALLOC					0x05	// Use 2K of Tx/Rx Buffer

#define WIZ_TX_BUF_ADDR  				0x4000  // W5100 send buffer base address
#define WIZ_RX_BUF_ADDR  				0x6000  // W5100 read buffer base address
#define WIZ_TX_BUF_MASK					0x07FF	// Tx 2K Buffer Mask:
#define WIZ_RX_BUF_MASK					0x07FF	// Rx 2K Buffer Mask:

//########################################################################################## WIZnet common registers 
// H_REGISTER
#define WIZ_COM_REG						0x00

// L_REGISTER 
#define WIZ_GTW_REG						0x01	// 0x01 to 0x04		(0..3)
#define WIZ_SUB_REG						0x05	// 0x05 to 0x08		(0..3)
#define WIZ_MAC_REG						0x09	// 0x09 to 0x0E		(0..5)
#define WIZ_IP_REG						0x0F	// 0x0F to 0x12		(0..3)

#define WIZ_RXMS_REG					0x1A	// RX Memory Size Register
#define WIZ_TXMS_REG					0x1B	// TX Memory Size Register
#define WIZ_INT_REG						0x15	// interrupt register
#define WIZ_INTM_REG					0x16	// interrupt mask register

//##########################################################################################
//########################################################################################## WIZnet socket registers 
// Socket H_REGISTER
#define S0_CR							0x04
#define S1_CR							0x05
#define S2_CR							0x06
#define S3_CR							0x07

// Socket L_REGISTER
#define S_MODE_REG						0x00	// mode register address		(MR)
#define S_COMMAND_REG					0x01	// command register address		(CR)
#define S_INTE_REG						0x02	// interrupt register address	(IR)
#define S_STATUS_REG					0x03	// status register address		(SR)
	
#define S_SOURCE_PORT					0x04	// 0x04 to 0x05		(0..1)
#define S_DESTINATION_MAC_ADDRESS		0x06	// 0x06 to 0x0B		(0..5)
#define S_DEST_IP_ADDR					0x0C	// 0x0C to 0x0F		(0..3)
#define S_DEST_PORT						0x10	// 0x10 to 0x11		(0..1)
#define S_MAX_SEGMENT_SIZE				0x12	// 0x12 to 0x13		(0..1)

#define S_TX_FREE_SIZE					0x20	// 0x20 to 0x21		(0..1)
#define S_TX_READ_POINTER				0x22	// 0x22 to 0x23		(0..1)
#define S_TX_WRITE_POINTER				0x24	// 0x24 to 0x25		(0..1)
#define S_RX_RECEIVED_SIZE				0x26	// 0x26 to 0x27		(0..1)
#define S_RX_READ_POINTER				0x28	// 0x28 to 0x29		(0..1)		

//########################################################################################## SOCKET_MR Mode
// Socket LL_REGISTER --> Content
#define MR_CLOSE						0x00		// unused socket
#define MR_TCP							0x01		// TCP
#define MR_UDP							0x02		// UDP
#define MR_IPRAW						0x03		// IP layer raw sock
#define MR_MACRAW						0x04		// MAC layer raw sock
#define MR_PPPOE						0x05		// PPPoE
#define MR_NO_DELAYED_ACK				0x20		// no delayed ACK(TCP) flag
#define MR_MULTI						0x80		// multicasting

//########################################################################################## SOCKET_CR Command
// Socket LL_REGISTER --> Content 
// after performing the commands, the register value will be automatically cleared to 0x00
#define CR_OPEN							0x01		// initialze or open socket
#define CR_LISTEN						0x02		// wait connection request in tcp mode (server mode)
#define CR_CONNECT						0x04		// send connection request in tcp mode (client mode)
#define CR_DISCONECT					0x08		// send closing request in tcp mode
#define CR_CLOSE						0x10		// close socket
#define CR_SEND							0x20		// update Tx memory pointer and send data
#define CR_SEND_MAC						0x21		// send data with MAC address, so without ARP process
#define CR_SEND_KEEP					0x22		// send keep alive message
#define CR_RECV							0x40		// update Rx memory buffer pointer and receive data

//########################################################################################## SOCKET interrupt
// Socket LL_REGISTER --> Content 
#define IR_CON							0x01		// connection is established
#define IR_DISCON						0x02		// connection termination is requested or finished
#define IR_RECV							0x04		// whenever W5100 receives data
#define IR_TIMEOUT						0x08		// if timeout occurs during connection establishment or termination and data transmission
#define IR_SEND_OK						0x10		// send operation is complete
#define IR_ACK							0x01		// write to clear the ir register

//########################################################################################## SOCKET_SR State 
#define SR_CLOSED						0x00		// closed
#define SR_INIT							0x13		// init state
#define SR_LISTEN						0x14		// listen state
#define SR_SYNSENT						0x15		// connection state (is automatically changed if connection is established)
#define SR_SYNRECV						0x16		// connection state (it normally respond to the request and changes to SOCK_ESTABLISH)
#define SR_ESTABLISHED					0x17		// success to connect
#define SR_FIN_WAIT						0x18		// closing state
#define SR_CLOSING						0x1A		// closing state
#define SR_TIME_WAIT					0x1B		// closing state
#define SR_CLOSE_WAIT					0x1C		// closing state
#define SR_LAST_ACK						0x1D		// closing state
#define SR_UDP							0x22		// UDP socket
#define SR_IPRAW						0x32		// IP raw mode socket
#define SR_MACRAW						0x42		// MAC raw mode socket
#define SR_PPPOE						0x5F		// PPPoE socket

//########################################################################################## Wiznet W5100 Op Code
#define WIZ_WRITE_OPCODE				0xF0
#define WIZ_READ_OPCODE					0x0F
	
	
	
//########################################################################################## Struct

typedef struct Wiz_and_tcp
{	
	uint8_t			bufRXTX[RX_BUF_MAX];		// Data buffer
	
	PORT_t			*hw_port;					// SPI Port
	SPI_Master_t	*hw_spi;					// SPI master Struct 	
	uint8_t			hw_pin_cs;					// Pin CS
	uint8_t			hw_pin_reset;				// Pin Reset
	
	uint8_t			tcp_mac_addr[6];			// Socket destination IP address
	uint8_t			tcp_ip_addr [4];			// IP address for WIZnet - base
	uint8_t			tcp_ip_dest [4];			// destination IP for WIZnet - base
	uint8_t			tcp_sub_mask[4];			// subnet mask for WIZnet
	uint8_t			tcp_gtw_addr[4];			// gateway for WIZnet
	uint16_t		tcp_port;
} WIZ_data_t;

	
//########################################################################################## Prototypes Of Global Functions

// Wiznet Config
void		wiz_init				(WIZ_data_t *wiz_data);
uint8_t		wiz_write				(WIZ_data_t *wiz_data, uint8_t addressH, uint8_t addressL, uint8_t data);
uint8_t		wiz_read				(WIZ_data_t *wiz_data, uint8_t addressH, uint8_t addressL);

// Wiznet Socket Communication
uint8_t		wiz_sock_connectServer	(WIZ_data_t *wiz_data, uint8_t sock, uint16_t dest_port);

uint8_t		wiz_sock_init			(WIZ_data_t *wiz_data, uint8_t sock, uint8_t ethProtocol, uint16_t tcpPort);
void		wiz_sock_close			(WIZ_data_t *wiz_data, uint8_t sock);
void		wiz_sock_connect		(WIZ_data_t *wiz_data, uint8_t sock);
void		wiz_sock_disconnect		(WIZ_data_t *wiz_data, uint8_t sock);

uint8_t		wiz_sock_listen			(WIZ_data_t *wiz_data, uint8_t sock);
uint8_t		wiz_sock_send			(WIZ_data_t *wiz_data, uint8_t sock);
uint8_t		wiz_sock_recv			(WIZ_data_t *wiz_data, uint8_t sock, uint8_t *data, uint16_t bufferLength);
uint16_t	wiz_sock_recvSize		(WIZ_data_t *wiz_data, uint8_t sock);

#endif 